/**
 * Copyright 2016 Rice University
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 *  the License. You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software distributed under the License is distributed
 *  on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for
 *  the specific language governing permissions and limitations under the License.
 */
part of default_controllers;

/**
 * A default IDE implementation. Consists of a toolbar, project explorer, console, and source edit areas.
 *
 * It is assumed that all files created in the editor are for the same programming language.
 */
class DefaultIdeController extends Controller
{
  /**
   * Controls the toolbar functionality of the IDE.
   */
  ToolbarController _toolbarController;

  /**
   * Controls the project explorer functionality of the IDE.
   */
  ProjectExplorerController _projectExplorerController;

  /**
   * Controls the source editor functionality of the IDE.
   */
  FileEditorController _sourceEditorController;

  /**
   * Controls the console functionality of the IDE.
   */
  ConsoleController _consoleController;

  /**
   * A factory used to create new source files in response to a "new file" request from the user.
   */
  NewSourceFileFactory _newSourceFileFactory;

  /**
   * The set of files open in the IDE.
   */
  Set<EditorFile> _sourceFiles = new Set();

  /**
   * The file currently displayed for editing.
   */
  EditorFile _primarySourceFile;

  /**
   * The language of all open files.
   */
  FileType _projectLanguage;

  /**
   * The language of all open files.
   */
  FileType get projectLanguage => _projectLanguage;

  /**
   * An optional message to print to the console when the IDE is started. Set a value of null for no message.
   */
  String _consoleInitialMessage;

  /**
   * Creates a new IDE.
   */
  factory DefaultIdeController(FileEditorView makeEditorView(SourceEditorViewModel), {ToolbarController toolbarController, ConsoleController consoleController,
                                ProjectExplorerController projectExplorerController, EditorFile initialFile,
                                String consoleInitialMessage, NewSourceFileFactory newFileFactory})
  {

    if(initialFile == null && newFileFactory == null)
    {
      newFileFactory = new NewSourceFileFactoryJava();
      initialFile = newFileFactory.makeSourceFile("Main");
    }
    else if(initialFile == null)
    {
      initialFile = newFileFactory.makeSourceFile("Main");
    }

    ContentMode editorMode = initialFile.fileType(
          (java) => ContentModeJava,
          (c) => ContentModeC,
          (js) => ContentModeJavaScript);


    FileEditorViewModel sourceEditorVm = new FileEditorViewModel(initialFile.contents, editorMode);
    FileEditorView sourceEditorView = makeEditorView(sourceEditorVm);
    FileEditorController sourceEditorController = new DefaultFileEditorController(sourceEditorVm, sourceEditorView);

    return new DefaultIdeController.allParams(toolbarController, consoleController, projectExplorerController,
                                              sourceEditorController, initialFile, consoleInitialMessage,
                                              newFileFactory);
  }

  /**
   * Creates a new IDE with the given values.  If any parameter is null, a default value will be selected
   * with the exception of [consoleInitialMessage] which will be interpreted as no message.
   */
  DefaultIdeController.allParams(ToolbarController toolbarController, ConsoleController consoleController,
                                 ProjectExplorerController projectExplorerController,
                                 FileEditorController sourceEditorController, EditorFile initialFile,
                                 String consoleInitialMessage, NewSourceFileFactory newFileFactory)
  {
    _sourceEditorController = sourceEditorController;

    _consoleInitialMessage = consoleInitialMessage;

    if(initialFile == null && newFileFactory == null)
    {
      newFileFactory = new NewSourceFileFactoryJava();
      initialFile = newFileFactory.makeSourceFile("Main");
    }
    else if(initialFile == null)
    {
      initialFile = newFileFactory.makeSourceFile("Main");
    }
    else if(newFileFactory == null) // match newFileFactory to the example language set by initialFile
    {
      newFileFactory = initialFile.fileType(
          (JavaLanguageType java) => new NewSourceFileFactoryJava(),
          (CLanguageType c) => new NewSourceFileFactoryC(),
          (JavaScriptLanguageType js) => new NewSourceFileFactoryJavaScript());
    }

    _newSourceFileFactory = newFileFactory;

    _projectLanguage = initialFile.fileType;

    if(toolbarController == null)
      toolbarController = _makeDefaultToolbarController();

    if(consoleController == null)
      consoleController = _makeDefaultConsoleController();

    if(projectExplorerController == null)
      projectExplorerController = _makeDefaultProjectExplorerController(initialFile);


    _toolbarController= toolbarController;
    _consoleController = consoleController;
    _projectExplorerController = projectExplorerController;
    _primarySourceFile = initialFile;
    _sourceFiles.add(_primarySourceFile);

  }

  /**
   * Used to populate _projectExplorerController if no value is given during construction.
   */
  DefaultProjectExplorerController _makeDefaultProjectExplorerController(EditorFile defaultSource)
  {
    ProjectExplorerViewModel explorerVm = new ProjectExplorerViewModel()
      ..addFile(defaultSource.path);
    ProjectExplorerView explorerView = new DefaultProjectExplorerView.defaultElementName(explorerVm);
    return new DefaultProjectExplorerController(explorerVm, explorerView);
  }


  /**
   * Used to populate _consoleController if no value is given during construction.
   */
  DefaultConsoleController _makeDefaultConsoleController()
  {
    return new DefaultConsoleController();
  }

  /**
   * Used to populate _toolbarController if no value is given during construction.
   */
  DefaultToolbarController _makeDefaultToolbarController()
  {
    return new DefaultToolbarController();
  }

  @override
  void start()
  {
    super.start(); // we want the "throw exception on double invocation" behavior

    /**
     * Wire up _toolbarController event handlers.
     */
    _toolbarController.onNewFileCreationRequested.listen( (_) { _handleNewFileCreationRequested(); });

    /**
     * Wire up _sourceEditorController event handlers.
     */
    _sourceEditorController.onContentsChanged.listen(_handleSourceChanged);
    _sourceEditorController.contents = getPrimarySourceFileContent();
    _sourceEditorController.fileType = _projectLanguage;

    /**
     * Wire up _projectExplorerController event handlers.
     */
    _projectExplorerController.onFileSelected.listen(_handleFileSelectedInProjectExplorer);

    /**
     * Start sub-controllers.
     */
    _sourceEditorController.start();
    _toolbarController.start();
    _projectExplorerController.start();
    _consoleController.start();

    /**
     * Print initial console message, if any.
     */
    if(_consoleInitialMessage != null)
      _consoleController.appendTextLine(_consoleInitialMessage);
  }

  /**
   * Update members to reflect new content for the primary source file.
   */
  void _handleSourceChanged(String newPrimarySourceContent)
  {
    _sourceFiles.remove(_primarySourceFile);
    _primarySourceFile = new EditorFile(_primarySourceFile.path, newPrimarySourceContent,
                                        _primarySourceFile.fileType);
    _sourceFiles.add(_primarySourceFile);
  }

  /**
   * Create and add a new file to the IDE.
   */
  void _handleNewFileCreationRequested()
  {

    /**
     * Decide the path of the new file.  Must not collide with another file's path in _sourceFiles
     */
    EditorFile proposedNewFile;
    {
      Set<String> existingFilePaths = _sourceFiles.map((EditorFile f) => f.path.basename);

      for(int i = 0; true; i++)
      {
        proposedNewFile = _newSourceFileFactory.makeSourceFile("File" + i.toString());

        if (!existingFilePaths.contains(proposedNewFile.path.basename))
          break;
      }
    }


    /**
     * Create/Set the new file as the primary file and update the project explorer and editor.
     */
    _primarySourceFile = proposedNewFile;
    _sourceFiles.add(_primarySourceFile);

    _projectExplorerController.addFile(_primarySourceFile.path);
    _sourceEditorController.contents = _primarySourceFile.contents;



  }

  /**
   * Change the file under edit in the editor to reflect the selection in the project explorer.
   */
  void _handleFileSelectedInProjectExplorer(String filename)
  {
    _primarySourceFile = _sourceFiles.where((EditorFile f) => f.path == filename).first;
    _sourceEditorController.contents = _primarySourceFile.contents;
  }

  /**
   * Gets the content of the file open for editing.
   */
  String getPrimarySourceFileContent()
  {
    return _primarySourceFile.contents;
  }

  /**
   * Removes all messages from the console.
   */
  void clearConsole()
  {
    _consoleController.clear();
  }

  /**
   * Prints the given formatted text to the console.
   */
  void appendHtmlToConsole(HtmlElement message)
  {
    _consoleController.appendHtml(message);
  }

  void appendTextLineToConsole(String message)
  {
    _consoleController.appendTextLine(message);
  }

  @override
  void suspend()
  {
    // do nothing
  }

  @override
  void resume()
  {
    // do nothing
  }





}