/**
 * Copyright 2016 Rice University
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 *  the License. You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software distributed under the License is distributed
 *  on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for
 *  the specific language governing permissions and limitations under the License.
 */
part of default_views;

/**
 * A basic project explorer view that shows a single list of files.
 *
 * Asssumes Font Awesome CSS icons are available. (http://fortawesome.github.io/Font-Awesome/)
 */
class DefaultProjectExplorerView extends ElementView implements ProjectExplorerView
{
  /**
   * The view model that the view reflects.
   */
  final ProjectExplorerViewModel _vm;

  @override
  Stream<ParsedPath> get onFileSelected => _onFileSelectedController.stream;

  /**
   * Controller for [onFileSelected]
   */
  StreamController<ParsedPath> _onFileSelectedController = new StreamController.broadcast();

  /**
   * The HTLM element used to render the list of files.
   */
  Element _explorerElement;

  /**
   * Creates a project explorer that renders a list of files into the HTML element with id
   * 'project-explorer'
   */
  DefaultProjectExplorerView.defaultElementName(ProjectExplorerViewModel vm)
      : this(vm, querySelector('#project-explorer'));

  /**
   * Creates a project explorer that renders a list of files into the given HTML element.
   */
  DefaultProjectExplorerView(this._vm, Element explorerElement) : super(_assertNonNull(explorerElement))
  {
    _explorerElement = explorerElement;
    _vm.onFilesAdded.listen(_handleOnFilesAdded);
  }

  /**
   * Helper function for [DefaultProjectExplorerView] that checks if the given argument for null.
   *
   * Throws [ArgumentError] with nice error message if null is given, else returns the argument.
   */
  static Element _assertNonNull(Element explorerElement)
  {
    if(explorerElement == null)
      new ArgumentError("explorerElement may not be null.");

    return explorerElement;
  }

  @override
  void show()
  {
    _reflectVm();
    super.show();
  }

  /**
   * Updates the view's rendering to reflect the current state of [_vm].
   */
  void _reflectVm()
  {
    _explorerElement.children.clear();

    /**
     * Sort the files into alphabetical order for presentation.
     */
    List<ParsedPath> filenamesPresentationOrder = new List()
      ..addAll(_vm.files)
      ..sort((ParsedPath a, ParsedPath b) => a.basename.compareTo(b.basename));

    /**
     * Create an HTML element for each file in the view model and append it to _explorerElement
     * in presentation order.
     */
    for(ParsedPath filename in filenamesPresentationOrder)
    {

      Element filenameElement;
      {

        SafeHtmlBuilder builder = new SafeHtmlBuilder()..appendEscaped(filename.toString());
        String htmlSafeName = builder.toSafeHtml().asString();

        Element nameAndIcon = new Element.html("<i></i>")
          ..classes.add("fa")       // dictated by font awesome
          ..classes.add("fa-file")  // dictated by font awesome for file icon
          ..innerHtml = "&nbsp;&nbsp;" + htmlSafeName;

        filenameElement = new DivElement()
          ..classes.add("project-explorer-file-entry")
          ..classes.add("unselectable")
          ..children.add(nameAndIcon);
      }

      filenameElement.onClick.listen( (MouseEvent e) { _handleFileElementDivClicked(filename); } );

      _explorerElement.children.add(filenameElement);

    }

  }

  /**
   * React to new files added to the view model by updating the rendering.
   */
  void _handleOnFilesAdded(Iterable<String> addedFiles)
  {
    _reflectVm();
  }

  /**
   * React to a file entry being clicked on the explorer by firing an event on [onFileSelected].
   */
  void _handleFileElementDivClicked(ParsedPath filename)
  {
    _onFileSelectedController.add(filename);
  }


}