/**
 * Copyright 2016 Rice University
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 *  the License. You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software distributed under the License is distributed
 *  on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for
 *  the specific language governing permissions and limitations under the License.
 */
part of view_models;

/**
 * Model of a single file's content under edit.
 */
class FileEditorViewModel
{
  /**
   * The file contents under edit.
   */
  String _contents = "";

  /**
   * The file contents under edit.
   */
  String get contents => _contents;

  /**
   * A flag to indicate if the file contents are modifiable.
   */
  bool _isReadOnly;

  /**
   * A flag to indicate if the file contents are modifiable.
   */
  bool get isReadOnly => _isReadOnly;

  /**
   * Sets the edibility mode of the editor.
   */
  void setReadOnly(bool readOnly) { _isReadOnly = readOnly; _onReadOnlyUpdatedController.add(_isReadOnly); }

  /**
   * The content mode of the editor.  I.e. what type of file is under edit? JavaScript source code? C source code?
   */
  ContentMode _contentMode;

  /**
   * The content mode of the editor.  I.e. what type of file is under edit? JavaScript source code? C source code?
   */
  ContentMode get contentMode => _contentMode;

  /**
   * Sets the content mode of the editor. I.e. What type of file is under edit.
   */
  void setContentMode(ContentMode contentMode)
    { _contentMode = contentMode; _onContentModeUpdatedController.add(_contentMode); }

  /**
   * Fired when the content mode of the editor is updated.
   */
  Stream<ContentMode> get onContentModeUpdated => _onContentModeUpdatedController.stream;

  /**
   * Backing controller for onContentModeUpdated
   */
  StreamController<ContentMode> _onContentModeUpdatedController = new StreamController.broadcast();

  /**
   * Fired when the contents of the editor are modified.
   */
  Stream<String> get onContentsChanged => _onContentsChangedController.stream;

  /**
   * Backing controller for onContentsChanged
   */
  StreamController<String> _onContentsChangedController = new StreamController.broadcast();

  /**
   * Fired when the read-only state of the editor ([isReadOnly]) is updated.  Message content is the current value
   * of [isReadOnly].
   *
   * Note: The event is fired for programmatic updates, not logical updates. A message may be generated indicating
   * the editor is in state A when the editor was already in state A.  The read-only state does not need to change
   * values for this event to fire.
   */
  Stream<bool> get onReadOnlyUpdated => _onReadOnlyUpdatedController.stream;

  /**
   * Backing controller for [onReadOnlyUpdated].
   */
  StreamController<bool> _onReadOnlyUpdatedController = new StreamController.broadcast();

  /**
   * Creates a new view model with the given contents and content mode.
   *
   * Optionally specify the initial read only state of the editor via [_isReadOnly].
   *
   * [_contents] and [_contentMode] may not be null.
   */
  FileEditorViewModel(this._contents, this._contentMode, [this._isReadOnly])
  {
    if(_contents == null)
      throw new ArgumentError.notNull("_source");

    if(_contentMode == null)
      throw new ArgumentError.notNull("_sourceLanguage");

    if(_isReadOnly == null)
      _isReadOnly = false;
  }

  /**
   * Sets the contents of the editor to the specified string content.
   *
   * Optionally suppress firing the [onContentsChanged] event by passing
   * [fireSourceChangedEvent] as [false].
   */
  void setContents(String newContents, [bool fireSourceChangedEvent = true])
  {
    String oldContents = _contents;

    _contents = newContents;

    if(oldContents != newContents)
      _onContentsChangedController.add(_contents);
  }

}

