/**
 * Copyright 2016 Rice University
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 *  the License. You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software distributed under the License is distributed
 *  on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for
 *  the specific language governing permissions and limitations under the License.
 */
part of vellum.views;

/**
 * [View] implementation for views that utilize an HTML element for rendering.
 *
 * Upon invocations of [show] the underlying HTML element's display CSS property is set to a user
 * defined value specified during construction.  Upon invocations of [hide] the property is set to 'none'.
 */
abstract class ElementView implements View
{
  /**
   * The HTML element used to render the view.
   */
  final Element _element;

  /**
   * The CSS display style to apply to [_element] when in the show state.
   */
  String _showDisplayCssStyle;

  /**
   * Creates an [ElementView] and configures it to use the given CSS style for its display property
   * when in the show state via the [showDisplayStyle] parameter. If [showDisplayStyle] is not specified
   * then the current display value on the element is used for the show state unless the 'none' is found.
   * In the case of 'none' the [showDisplayStyle] is defaulted to 'inline'.
   *
   */
  ElementView(this._element, [String showDisplayStyle])
  {
    if(showDisplayStyle == null)
      showDisplayStyle = _element.style.display == "none" ? "inline" : _element.style.display;

    _showDisplayCssStyle = showDisplayStyle;

  }

  @override
  void show()
  {
    _element.style.display = _showDisplayCssStyle;
  }

  @override
  void hide()
  {
    _element.style.display = "none";
  }
}