/**
 * Copyright 2016 Rice University
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 *  the License. You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software distributed under the License is distributed
 *  on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for
 *  the specific language governing permissions and limitations under the License.
 */
part of vellum.views;

/**
 * [FileEditorViewExternal] instances often serve as proxy objects for JavaScript editor implementations that act as
 * the actual file editors in the browser.  As such FileEditorViewExternal objects may become out of sync with
 * the state of the external JavaScript implementations during execution without coordination between the Dart
 * and JavaScript layers.
 *
 * This class provides a mechanism by which messages about state changes originating in the JavaScript layer
 * can be routed back to FileEditorViewExternal instances via the [register] method.
 *
 * After registration, JavaScript code may invoke the JavaScript functions [handleCursorUpdated] and
 * [handleContentsUpdated] to notify their corresponding proxy instances of changes originating in the JavaScript layer.
 *
 */
class JavaScriptEditorViewRegistrar
{
  /**
   * Maps each [FileEditorViewExternal]'s unique id onto the instance itself.
   */
  static Map<Object,FileEditorViewExternal> _fileEditorIdToOwner = { };

  /**
   * Flag to trach whether [initialize()] has been run.
   */
  static bool _isClassInitialized = false;

  /**
   * Static initializer for the JavaScriptEditorViewRegistrar class.
   *
   * Should be called explicitly from each function in [JavaScriptEditorViewRegistrar]
   * since Dart does not have language level support for static class initializers.
   */
  static void initialize()
  {
    if(!_isClassInitialized)
    {
      js.context['handleCursorUpdated'] = _handleCursorUpdatedFromJavaScript;
      js.context['handleContentsUpdated'] = _handleContentsUpdatedFromJavaScript;
      _isClassInitialized = true;
    }
  }

  /**
   * Registers the given [FileEditorViewExternal] for state change callbacks from JavaScript.
   */
  static void register(FileEditorViewExternal view)
  {
    if(!_isClassInitialized)
    {
      initialize();
    }

    _fileEditorIdToOwner[view.id] = view;
  }

  /**
   * Notifies the editor specified by the given id that the cursor row and column may have changed
   * as a result of some action taken outside of this framework.
   *
   * Exposed to JavaScript callers as [handleCursorUpdated]
   *
   * Throws [UnregisteredFileEditorIdException] if the given key does not map to a registered file editor.
   */
  static void _handleCursorUpdatedFromJavaScript(Object fileEditorId, int cursorRow, int cursorColumn)
  {
    if(!_isClassInitialized)
    {
      initialize();
    }

    if(!_fileEditorIdToOwner.containsKey(fileEditorId))
      throw new UnregisteredFileEditorIdException(fileEditorId);

    _fileEditorIdToOwner[fileEditorId].handleCursorUpdatedExternally(cursorRow, cursorColumn);
  }

  /**
   * Notifies the editor specified by the given id that the cursor row and column may have changed
   * as a result of some action taken outside of this framework.
   *
   * Exposed to JavaScript callers as [handleCursorUpdated]
   *
   * Throws [UnregisteredFileEditorIdException] if the given key does not map to a registered file editor.
   */
  static void _handleContentsUpdatedFromJavaScript(Object fileEditorId, String newSource)
  {
    if(!_isClassInitialized)
    {
      initialize();
    }

    if(!_fileEditorIdToOwner.containsKey(fileEditorId))
      throw new UnregisteredFileEditorIdException(fileEditorId);

    _fileEditorIdToOwner[fileEditorId].handleContentsUpdatedExternally(newSource);

  }

}

/**
 * Thrown to indicate a file editor id was provided that does not map to a registered editor.
 */
class UnregisteredFileEditorIdException implements Exception
{
  final String fileEditorId;

  UnregisteredFileEditorIdException(this.fileEditorId);
}

