/**
 * Copyright 2016 Rice University
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 *  the License. You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software distributed under the License is distributed
 *  on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for
 *  the specific language governing permissions and limitations under the License.
 */
part of vellum.views;

/**
 * A [FileEditorView] implementation backed by external JavaScript code that defines the following global
 * functions to provide file editor capability:
 *
 * function setEditorSourceContent(editorId, String sourceContent) { ... }
 *
 * function setSourceLineBackgroundEffect(editorId, lineNumber, color) { ... }
 *
 * function setEditorPaneReadOnly(editorId) { ... }
 *
 */
abstract class JavaScriptFileEditorView implements FileEditorViewExternal
{
  /**
   * The view model reflected by the view.
   */
  final FileEditorViewModel _vm;

  /**
   * Controller for [onFocusedLineChanged].
   */
  final StreamController<int> _onFocusedLinedChangedController = new StreamController.broadcast();

  @override
  Stream<int> get onFocusedLineChanged => _onFocusedLinedChangedController.stream;

  /**
   * Controller for [onContentsChanged].
   */
  final StreamController<String> _onContentsChangedController = new StreamController.broadcast();

  @override
  Stream<String> get onContentsChanged => _onContentsChangedController.stream;

  /**
   * Flag to indicate whether [show] has ever been invoked.
   */
  bool _everShown = false;

  /**
   * The last cursor row value reported to the view via the [handleCursorUpdatedExternally] method.
   */
  int _lastSeenCursorRow = null;

  /**
   * The last contents value sent to or received from the JavaScript codebase.
   */
  String _externalContentsState;

  /**
   * Constructs a new view reflecting the given view model.
   */
  JavaScriptFileEditorView(this._vm)
  {
    _vm.onContentsChanged.listen(_handleContentsChangedFromVm);
    _vm.onReadOnlyUpdated.listen(_handleReadOnlyUpdated);
  }

  @override
  void show()
  {
    if(!_everShown) // if first time shown, register for callbacks from the JavaScript layer
    {
      // n.b. this is done in show() to give subclass constructors an opportunity to set the id member
      // prior to invocation of register().  Placing register() in the constructor has register() invoked
      // while id is null.
      JavaScriptEditorViewRegistrar.register(this);
      _everShown = true;
    }

    _reflectVm();
  }

  /**
   * Ensure the rendered view reflects the state of the view model.
   */
  void _reflectVm()
  {
    if(!_everShown) // if not shown yet, there is no view to reflect the vm state.
      return;

    if(_externalContentsState != _vm.contents) // set contents state in the JavaScript layer if it does not match the vm
    {
      // only setting when the states mismatch avoid potentially expensive operations in the JS to render
      // what is already currently rendered.
      setFileEditorContents(this.id, _vm.contents);
      _externalContentsState = _vm.contents;
    }

    if (_vm.isReadOnly)
      setEditorPaneReadOnly(this.id);

  }

  /**
   * On changes to the file contents driven from the VM and reflect those changes in the view.
   */
  void _handleContentsChangedFromVm(String newContents)
  {
    _reflectVm();
  }

  /**
   * On changes to the editor read only state driven from the VM and reflect those changes in the view.
   */
  void _handleReadOnlyUpdated(bool currentValue)
  {
    _reflectVm();
  }



  @override
  void handleCursorUpdatedExternally(int cursorRow, int cursorColumn)
  {

    if(_lastSeenCursorRow != cursorRow)
    _onFocusedLinedChangedController.add(cursorRow);

    _lastSeenCursorRow = cursorRow;
  }

  @override
  void handleContentsUpdatedExternally(String newContents)
  {
    _externalContentsState = newContents;

    if(_vm.contents != newContents)
    {
      _onContentsChangedController.add(newContents);

    }
  }
}

/**
 * Set the file editor contents of editor [editorId] to [contents] in the JavaScript layer.
 */
@JS("")
external void setFileEditorContents(Object editorId, String contents);

/**
 * Set the file editor with [editorId] to to the read only in the JavaScript layer.
 */
@JS("")
external void setEditorPaneReadOnly(Object editorId);